#!/usr/bin/python
# $Header: /home/guyru/cvsroot/css-scripts/csscompactor.py,v 1.7 2006/12/20 07:17:06 guyru Exp $
"""CSS filesize reducer
Author: Guy Rutenberg
License: GPLv2
Version: 0.4
This script takes a css file and compacts it. It won't compress it it will reduce file size by removing blanks and unnecessary whitespace. The script will cause the css file to be almost unreadble for human eyes but it will still be understandable for computers.

CHANGE LOG
csscompactor-0.4:
- Operate on file lists.
- Some bugfixes related to file handling.
- Better error description
"""

###########################################################################
 #   Copyright (C) 2006-2007 by Guy Rutenberg                              #
 #   guyrutenberg@gmail.com                                                #
 #                                                                         #
 #   This program is free software; you can redistribute it and/or modify  #
 #   it under the terms of the GNU General Public License as published by  #
 #   the Free Software Foundation; either version 2 of the License, or     #
 #   (at your option) any later version.                                   #
 #                                                                         #
 #   This program is distributed in the hope that it will be useful,       #
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
 #   GNU General Public License for more details.                          #
 #                                                                         #
 #   You should have received a copy of the GNU General Public License     #
 #   along with this program; if not, write to the                         #
 #   Free Software Foundation, Inc.,                                       #
 #   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
############################################################################

import sys
import os
from optparse import OptionParser

def main():
	#parse the comandline
	usage = "usage: %prog [options] FILENAME" # the usage string for OptionParser
	version="%prog 0.4"
	parser = OptionParser(usage=usage, version=version, description="A script that reduce css files and restores them to readable form")
	parser.add_option("-c", "--compact",
		type="int", dest="compact", default=1, metavar="N",
		help="tells the script how much it should try to reduce the filesize. 1 - removes all whitespaces, newlines and comments. 2 - removes only comments [default=%default] ")
	parser.add_option("-u", "--uncompact",
		action="store_true", default=False, dest="uncompact",
		help="restores the css file to a human readable text.")
	parser.add_option("-o", "--ouput",
		type="string", dest="output", metavar="FILE",
		help="tells the script to which file it should write the output. [default: the input filename]")
	parser.add_option("-l", "--filelist", action="store_true", default=False, dest="filelist",
		help="Consider FILENAME to be a list of files to compact. Filenames should be seperated by newlines, lines that start with # are ignored")
	parser.add_option("-s", "--strip", type="int", dest="strip", default=0, metavar="N",
		help="When reading from a file list, strip N chars from the file name when writing thw output.")
	(options, args) = parser.parse_args()
	
	if len(args)<1:
		parser.error("You must supply an input filename")
	if len(args)>1:
		print "Ignoring arguments: ", ", ".join(["%s" % (arg,) for arg in args[1:]])
	
	if options.filelist: #handle file lists
		try:
			f = open(args[0], 'r')
		except IOError:
			print "Couldn't open file list", args[0]
			sys.exit(1)
		for line in f.readlines():
			line=line.strip()
    			if line[0]=='#': #skip lines if needed
				continue
			
			try:
				file = open (line, 'r')
			except IOError, (errno, strerror):
				print "Couldn't open file ", line, (errno, strerror)
				continue
			
			content = file.read()
			file.close()
			
			if options.uncompact:
				content = uncompact(content)
			else:
				content = compact(content, options.compact)
				
			try:
				if options.strip>0:
					file = open (line[0:-(options.strip)], 'w')
				else:
					file = open (line, 'w')
			except IOError, (errno, strerror):
				print "Couldn't write to file  ", line[0:-(strip+1)], (errno, strerror)
				
			file.write(content)
			file.close()
		f.close()
	else:
		if options.output==None:
			options.output=args[0];
			
		try:
			file = open (args[0], 'r')
		except IOError, (errno, strerror):
			print "Couldn't open file ", args[0], (errno, strerror)
			sys.exit(1)
		
		content = file.read()
		file.close()
		
		if options.uncompact:
			content = uncompact(content)
		else:
			content = compact(content, options.compact)
		
		try:
			file = open (options.output, 'w')
		except IOError, (errno, strerror):
			print "Couldn't write to file  ", options.output, (errno, strerror)
			
		file.write(content)
		file.close()

def compact(content, level=1):
	"""This is the main function responsible for cutting down the file size.
	@type content: string
	@param content: holds the string that should be compacted.
	@type level: int
	@param level: tells the function how to strip the file. 
	@rtype: string
	@return: the compacted string."""
	
	newcontent='' # holds the compacted string
	temp=''
	iscomment=False
	
	if level==1 or level==2: #remove all whithspaces and  comments.
		for char in content:
			#begin remove comments section
			if char=='*' and iscomment:
				temp='*'
				continue
			if temp=='*':	#first you need to check for ending a comment;
				if char == '/': #should only be here if in comment any way so no need to check again
					iscomment=False
					temp=''
					continue
				else: #no writing to the output since we should be in comment
					temp=''
			if char=='*' and iscomment:
				temp='*'
				continue
				
			if iscomment:
				continue
			if char=='/':
				temp='/'
				continue #return to the beginning of the loop
			if temp=='/':
				if char=='*':
					iscomment=True
					temp=''
					continue
				else:
					newcontent += temp
					temp=''
			#end remove comments section
			
			#handle spaces
			if level == 1:
				if temp in (';', ':', ',', '{', '}'):
					if char == ' ':
						continue #move on to the next char
					else:
						temp='' #reset temp and continue processing the char
				
				if char in (';', ':', ',', '{', '}'):
					temp = char;#continue processing the char
					
				if char == ' ': #handle special cases where we should check after the space
					temp=' '
					continue
				if temp==' ':
					if char in ('}', '{', ';', ':'): # we should also find a way to check when we could delete a space before a dot
						temp=''
						continue
					else:
						newcontent+=temp
						temp=''
			#end space handling
			
			if (char not in ('\n', '\r', '\t')) or level==2: 
				newcontent += char

	return newcontent
def uncompact(content):
	"""This is the main function responsible for restoring the file to a human readble state
	@type content: string
	@param content: holds the string that should be uncompacted.
	
	@rtype: string
	@return: the compacted string."""
	newcontent=''
	indent = False
	
	for char in content:
		if indent and (newcontent[-1]=='\n'):
			newcontent += '\t'
		newcontent += char
		if char==';':
			newcontent+='\n'
		if char=='{':
			newcontent+='\n'
			indent = True
		if char=='}':
			newcontent = newcontent[0:-2]+'}\n'
			indent = False
		if char in (':',','):
			newcontent += ' '
		
		
	return newcontent
	
if __name__=="__main__":
	main()